/// <reference types="react" />
import { Method, AxiosHeaders, AxiosResponse, AxiosError } from 'axios';
import React$1 from 'react';

interface ApiRequestConfig {
    url: string;
    method?: Method;
    baseURL?: string;
    headers?: AxiosHeaders;
    params?: any;
    timeout?: number;
    timeoutErrorMessage?: string;
    transformResponse?: (response: AxiosResponse) => any;
    transformError?: (error: AxiosError) => any;
}
interface ApiUploadConfig {
    file: File;
    cdnUrl?: string;
    headers?: AxiosHeaders;
    params?: any;
    timeout?: number;
    timeoutErrorMessage?: string;
    uploadFileType?: 'form-data' | 'binary';
    transformResponse?: (response: AxiosResponse) => any;
    transformError?: (error: AxiosError) => any;
}
interface ApiProviderProps {
    config: {
        baseURL?: string;
        token?: string;
        solrToken?: string;
        headers?: AxiosHeaders;
    };
    children: React.ReactNode;
}

declare const API: {
    request: (config: ApiRequestConfig) => Promise<any>;
    solr: (config: ApiRequestConfig) => Promise<any>;
    upload: (config: ApiUploadConfig) => Promise<any> | undefined;
};

declare const ApiProvider: React$1.FC<ApiProviderProps>;

export { API, ApiProvider };
