import axios from 'axios';
import React, { createContext } from 'react';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol */


function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

let ApiContext = null;
const ApiProvider = (props) => {
    const { children, config } = props;
    ApiContext = createContext(config);
    return React.createElement(ApiContext.Provider, { value: config }, children);
};

const API = {
    request: (config) => {
        const defaultConfig = (ApiContext === null || ApiContext === void 0 ? void 0 : ApiContext._currentValue) || {};
        const { baseURL: defaultBaseURL, token, headers: defaultHeader = {} } = defaultConfig;
        const { method = 'GET', url, baseURL, headers = {}, params, timeout = 10000, timeoutErrorMessage = 'Hệ thống không phản hồi. Vui lòng thử lại sau', transformResponse, transformError } = config;
        const configHeader = token ? Object.assign({ Authorization: `Bearer ${token}` }, defaultHeader) : defaultHeader;
        const requestConfig = {
            method,
            url,
            baseURL: baseURL || defaultBaseURL,
            headers: Object.assign(Object.assign({ 'Content-Type': 'application/json' }, configHeader), headers),
            timeout,
            timeoutErrorMessage
        };
        if (params) {
            method.toLowerCase().trim() === 'get' ? (requestConfig.params = params) : (requestConfig.data = params);
        }
        return axios(requestConfig)
            .then((response) => {
            const { data } = response;
            return transformResponse ? transformResponse(response) : data;
        })
            .catch((error) => {
            var _a;
            if (transformError) {
                return Promise.reject(transformError(error));
            }
            return Promise.reject(((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.data) || error);
        });
    },
    solr: (config) => {
        const defaultConfig = (ApiContext === null || ApiContext === void 0 ? void 0 : ApiContext._currentValue) || {};
        const { baseURL: defaultBaseURL, solrToken, headers: defaultHeader = {} } = defaultConfig;
        const { method = 'GET', url, baseURL, headers = {}, params, timeout = 10000, timeoutErrorMessage = 'Hệ thống không phản hồi. Vui lòng thử lại sau', transformResponse, transformError } = config;
        const configHeader = solrToken
            ? Object.assign({ Authorization: `Basic ${solrToken}` }, defaultHeader) : defaultHeader;
        let urlRequest = url;
        if (method === 'GET') {
            urlRequest = `${url}?${(params === null || params === void 0 ? void 0 : params.q) ? '' : '&q=*%3A*'}&omitHeader=true`;
        }
        if (['POST', 'PUT', 'DELETE', 'POST_MULTI', 'PUT_MULTI'].includes(method)) {
            urlRequest = `${url}?commit=true`;
        }
        const requestConfig = {
            method: method === 'GET' ? 'GET' : 'POST',
            url: urlRequest,
            baseURL: baseURL || defaultBaseURL,
            headers: Object.assign(Object.assign({ 'Content-Type': 'application/json' }, configHeader), headers),
            timeout,
            timeoutErrorMessage
        };
        if (method === 'GET' && params) {
            requestConfig.params = params;
        }
        else {
            let requestData = null;
            if (method === 'POST') {
                requestData = { add: [params] };
            }
            if (['POST_MULTI', 'PUT_MULTI'].includes(method)) {
                requestData = { add: params };
            }
            if (method === 'PUT') {
                const { id } = params, rest = __rest(params, ["id"]);
                const dataTransform = Object.entries(rest).reduce((prev, curr) => {
                    const [field, value] = curr;
                    const obj = {};
                    obj[field] = { set: value };
                    return Object.assign(Object.assign({}, prev), obj);
                }, {});
                requestData = { add: [Object.assign({ id }, dataTransform)] };
            }
            if (method === 'DELETE') {
                let deleteIds = [];
                if (Array.isArray(params.id) && params.id.length) {
                    deleteIds = params.id;
                }
                else {
                    deleteIds = [params.id];
                }
                requestData = { delete: deleteIds };
            }
            requestConfig.data = requestData;
        }
        return axios(requestConfig)
            .then((response) => {
            var _a;
            const { data } = response;
            return transformResponse ? transformResponse(response) : (_a = data === null || data === void 0 ? void 0 : data.response) === null || _a === void 0 ? void 0 : _a.docs;
        })
            .catch((error) => {
            var _a, _b;
            if (transformError) {
                return Promise.reject(transformError(error));
            }
            const errorResponse = ((_b = (_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.error) || error;
            return Promise.reject(Object.assign(Object.assign({}, errorResponse), { message: errorResponse === null || errorResponse === void 0 ? void 0 : errorResponse.msg }));
        });
    },
    upload: (config) => {
        const defaultConfig = (ApiContext === null || ApiContext === void 0 ? void 0 : ApiContext._currentValue) || {};
        const { cdnURL: defaultCdnURL, token, headers: defaultHeader = {} } = defaultConfig;
        const configHeader = token ? Object.assign({ Authorization: `Bearer ${token}` }, defaultHeader) : defaultHeader;
        const { file, uploadFileType = 'form-data', cdnUrl, headers = {}, params, timeout = 10000, timeoutErrorMessage = 'Hệ thống không phản hồi. Vui lòng thử lại sau', transformError, transformResponse } = config;
        if (!file) {
            return Promise.resolve(null);
        }
        const uploadUrl = cdnUrl || defaultCdnURL;
        const requestConfig = {
            headers: Object.assign(Object.assign({ 'Content-Type': 'multipart/form-data' }, configHeader), headers),
            params,
            timeout,
            timeoutErrorMessage
        };
        if (uploadFileType === 'binary') {
            const reader = new FileReader();
            reader.readAsArrayBuffer(file);
            reader.onload = () => __awaiter(void 0, void 0, void 0, function* () {
                const blob = new Blob([reader.result]);
                return axios
                    .put(uploadUrl, blob, requestConfig)
                    .then((response) => {
                    const { data } = response;
                    return transformResponse ? transformResponse(response) : data;
                })
                    .catch((error) => {
                    var _a;
                    if (transformError) {
                        return Promise.reject(transformError(error));
                    }
                    return Promise.reject(((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.data) || error);
                });
            });
            return;
        }
        const formData = new FormData();
        formData.append('file', file);
        return axios
            .post(uploadUrl, formData, requestConfig)
            .then((response) => {
            const { data } = response;
            return transformResponse ? transformResponse(response) : data;
        })
            .catch((error) => {
            var _a;
            if (transformError) {
                return Promise.reject(transformError(error));
            }
            return Promise.reject(((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.data) || error);
        });
    }
};

export { API, ApiProvider };
//# sourceMappingURL=index.esm.js.map
